/*
nzev: client.cpp
autor: Pavel Hasenhrl
datum: 17.5.2009
verze: 1.0
*/

#include "client.h"
#include <string>

#ifdef MYDEBUG
	#include <iostream>
	#include <iomanip>
#endif

using namespace std;
using namespace Exchanger;

const char CClientPeer::CLIENT_CHALLENGE[] = "openmachi challenge v1.0";

CClientPeer::CClientPeer(SOCKET _socket, const sockaddr_in &_addr, CCrypto *_crypt, const char *_ntwrk)
{
	m_buffer = NULL;
	m_socket = _socket;
	m_addr = _addr;
	m_crypt = _crypt;
	m_pass = NULL;
	SetNetwork(_ntwrk);
}

CClientPeer::~CClientPeer()
{
	if(m_socket) {
		shutdown(m_socket, 1);
		closesocket(m_socket);
	}
	if(m_buffer)
		delete [] m_buffer;
	if(m_pass)
		delete [] m_pass;
	if(m_name)
		delete [] m_name;
	if(m_ntwrk)
		delete [] m_ntwrk;
}

bool CClientPeer::Shutdown()
{
	if(m_err != ET_SOCKET_ERROR)
		shutdown(m_socket, 1);
	m_err = ET_CLOSED;

	char buffer[20];
	int ret = recv(m_socket, buffer, sizeof(buffer), 0);
	if(ret == 0 || ret == SOCKET_ERROR)
		return true;
	return false;
}

void CClientPeer::SetPasswd(const char *_pass)
{
	size_t size = strlen(_pass) + 1;
	m_pass = new char[size];
	memcpy(m_pass, _pass, size);
}

void CClientPeer::SetNetwork(const char *_ntwrk)
{
	size_t size = strlen(_ntwrk) + 1;
	m_ntwrk = new char[size];
	memcpy(m_ntwrk, _ntwrk, size);
}

bool CClient::Connect(const char *_addr, unsigned short _port)
{
	m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(m_socket == INVALID_SOCKET) {	
		m_socket = 0;
		m_err = ET_SOCKET_ERROR;
		return false;
	}
	hostent *host;
	host = gethostbyname(_addr);
 	if(host) {
		setsockopt(m_socket, SOL_SOCKET, SO_OOBINLINE, FALSE, sizeof(BOOL));
		sockaddr_in svr_addr;
		memcpy(&(svr_addr.sin_addr), host->h_addr_list[0], host->h_length);
		svr_addr.sin_family = AF_INET;
		svr_addr.sin_port = htons(_port);
		if(connect(m_socket, (sockaddr*)&svr_addr, sizeof(svr_addr)) == 0) {
			m_addr = svr_addr;
			m_state = CS_NAME;
			return true;
		}
	}
	closesocket(m_socket);
	m_socket = 0;
	m_err = ET_SOCKET_ERROR;
	return false;
}

bool CClient::Exchange(const char *_pass, const char *_name, const char *_ntwrk)
{
	SetPasswd(_pass);
	SetNetwork(_ntwrk);
	m_name = NULL;
	m_size = (unsigned int)strlen(_name) + 1;
	m_name = new char[m_size];
	memcpy(m_name, _name, m_size);
	m_remain = m_size;
	m_state = CS_NAME;
	bool ret = false;

	while(1) {
		if(Process())
			continue;
		switch(GetLastError()) {
		case ET_NOT_ALL_DATA:
			continue;
		case ET_COMPLETED:
			ret = true;
			break;
		}

		if(Shutdown())
			break;
	}

	return ret;
}

bool CClient::Process()
{
//	char buffer[255];
	string buffer;
//	*buffer = 0; // debug
	switch(m_state) {
	case CS_NAME: // putname
		if(SendName()) {
			m_state = CS_CHALLENGE;
			return true;
		}
		break;
	case CS_CHALLENGE:
		if(SendChallenge()) {
			m_state = CS_CA;
			return true;
		}
		break;
	case CS_CA:
		/*strcpy_s(buffer, sizeof(buffer) - 1, m_ntwrk);
		strcat_s(buffer, sizeof(buffer) - 1, "\\ca.crt");*/
		buffer = m_ntwrk;
		buffer += "\\ca.crt";
		if(RecvFile(buffer.c_str())) {
			m_state = CS_CSR;
			return true;
		}
		break;
	case CS_CSR:
		/*strcpy_s(buffer, sizeof(buffer) - 1, m_ntwrk);
		strcat_s(buffer, sizeof(buffer) - 1, "\\client.csr");*/
		buffer = m_ntwrk;
		buffer += "\\client.csr";
		if(SendFile(buffer.c_str())) {
			m_state = CS_CRT;
			return true;
		}
		break;
	case CS_CRT:
		/*strcpy_s(buffer, sizeof(buffer) - 1, m_ntwrk);
		strcat_s(buffer, sizeof(buffer) - 1, "\\client.crt");*/
		buffer = m_ntwrk;
		buffer += "\\client.crt";
		if(RecvFile(buffer.c_str())) {
			m_state = CS_SHUTDOWN;
			m_err = ET_COMPLETED;
		}
		break;
	case CS_SHUTDOWN:
		return false;
	}

	#ifdef MYDEBUG
		if(m_name)
			cout << m_name << endl;
		else
			cout << "unnamed" << endl;
		cout << buffer << " - ";
		switch(m_err) {
		case ET_FILE_ERROR:
			cout << "file_error" << endl;
			break;
		case ET_SOCKET_ERROR:
			cout << "socket_error" << endl;
			break;
		case ET_SOCKET_CLOSED:
			cout << "socket_closed" << endl;
			break;
		case ET_SIGN_ERROR:
			cout << "sign_error" << endl;
			break;
		case ET_CRYPT_ERROR:
			cout << "crypt_error" << endl;
			break;
		case ET_NOT_ALL_DATA:
			cout << "not_all_data" << endl;
			break;
		case ET_NAME_TOO_LONG:
			cout << "name_too_long" << endl;
			break;
		case ET_TIMEOUT:
			cout << "timeout" << endl;
			break;
		case ET_COMPLETED:
			cout << "completed" << endl;
			break;
		}
	#endif

	return false;
}

CServerClient::CServerClient(SOCKET _socket, const sockaddr_in _addr, CCrypto *_cprov, const CConfig *_conf, bool (* _SignCSR)(const CConfig *, const char *)): CClientPeer(_socket, _addr, _cprov, _conf->GetEntryValueA(ET_SESSION))
{
	m_SignCSR = _SignCSR;
	m_addr = _addr;
	m_name = NULL;
	m_conf = _conf;
	m_state = CS_NAME;
	m_size = m_remain = CLIENT_MAX_NAME;
	m_buffer = new char[m_size];
}

void CServerClient::fdadd(fd_set *_read, fd_set *_write)
{
	switch(m_state) {
	case CS_NAME:
	case CS_CHALLENGE:
	case CS_CSR:
	case CS_SHUTDOWN:
		if(_read)
			FD_SET(m_socket, _read);
		break;
	case CS_CA:
	case CS_CRT:
		if(_write)
			FD_SET(m_socket, _write);
		break;
	}
}

/*
v m_size velikost _data
*/
bool CClientPeer::SendData(const char *_data)
{
	if(!m_buffer) {
		m_buffer = new char[m_size + sizeof(u_long)];
		memcpy(m_buffer + sizeof(u_long), _data, m_size);
		*(u_long *)m_buffer = htonl((u_long)m_size);

		m_size += sizeof(u_long);
		m_remain = m_size;
	}

	int ret;
	ret = send(m_socket, m_buffer + (m_size - m_remain), m_remain, 0);
	if(ret == SOCKET_ERROR) {
		m_err = ET_SOCKET_ERROR;
		return false;
	}
	m_remain -= ret;

	if(m_remain) {
		m_err = ET_NOT_ALL_DATA;
		return false;
	}

	delete [] m_buffer;
	m_buffer = NULL;
	m_size = 0;
	return true;
}

bool CClientPeer::RecvData()
{
	int ret;
	if(!m_buffer) {
		ret = recv(m_socket, (char *)&m_size, sizeof(u_long), MSG_PEEK);
		if(ret == 0) {
			m_err = ET_SOCKET_CLOSED;
			return false;
		}
		if(ret == SOCKET_ERROR) {
			m_err = ET_SOCKET_ERROR;
			return false;
		}
		if(ret != sizeof(u_long)) {
			m_err = ET_NOT_ALL_DATA;
			return false;
		}
		recv(m_socket, (char *)&m_size, sizeof(u_long), 0);
		m_size = ntohl((u_long)m_size);

		m_buffer = new char[m_size];
		m_remain = m_size;
	}

	ret = recv(m_socket, m_buffer + (m_size - m_remain), m_remain, 0);
	if(ret == 0) {
		m_err = ET_SOCKET_CLOSED;
		return false;
	}
	if(ret == SOCKET_ERROR) {
		m_err = ET_SOCKET_ERROR;
		return false;
	}
	m_remain -= ret;

	if(m_remain) {
		m_err = ET_NOT_ALL_DATA;
		return false;
	}
	return true;
}

bool CClientPeer::SendChallenge()
{
	const char *ct = NULL;
	if(!m_buffer) {
		m_size = (unsigned int)strlen(CLIENT_CHALLENGE);
		ct = m_crypt->EncryptData(m_pass, CLIENT_CHALLENGE, m_size);
		if(!ct) {
			m_err = ET_CRYPT_ERROR;
			return false;
		}
	}
	return SendData(ct);
}

bool CClientPeer::RecvChallenge()
{
	if(RecvData()) {
		const char *pt;
		pt = m_crypt->DecryptData(m_pass, m_buffer, m_size);
		delete [] m_buffer;
		m_buffer = NULL;

		if(!pt)
			m_err = ET_CRYPT_ERROR; // nepodarilo se dekryptovat data
		else if(strlen(CLIENT_CHALLENGE) != m_size || strncmp(pt, CLIENT_CHALLENGE, m_size))
			m_err = ET_WRONG_PASSWORD;
		else
			return true;
	}
	return false;
}

bool CClientPeer::SendFile(const char *_file)
{
	const char *ct = NULL;
	if(!m_buffer) {
		ct = m_crypt->EncryptFile(m_pass, _file, m_size);
		if(!ct) {
			m_err = ET_CRYPT_ERROR;
			return false;
		}
	}
	return SendData(ct);
}

bool CClientPeer::RecvFile(const char *_file)
{
	if(RecvData()) {
		bool ret;
		ret = m_crypt->DecryptFile(m_pass, _file, m_buffer, m_size);
		delete [] m_buffer;
		m_buffer = NULL;

		if(ret)
			return true;

		m_err = ET_CRYPT_ERROR; // nepodarilo se dekryptovat data
	}
	return false;
}

/*
m_name = NULL
m_size = m_remain
m_buffer[m_size]
*/
bool CServerClient::RecvName()
{
	int ret;
	ret = recv(m_socket, m_buffer + (m_size - m_remain), m_remain, 0);
	if(ret == 0) {
		m_err = ET_SOCKET_CLOSED;
		return false;
	}
	if(ret == SOCKET_ERROR) {
		m_err = ET_SOCKET_ERROR;
		return false;
	}
	m_remain -= ret;

	char *pos = (char *)memchr(m_buffer, 0, m_size - m_remain);
	if(pos) {
		ret = (pos - m_buffer) + 1;
		m_name = new char[ret];
		memcpy(m_name, m_buffer, ret);
		delete[] m_buffer;
		m_buffer = NULL;
		return true;
	} else if(!m_remain) {
		m_err = ET_NAME_TOO_LONG;
		return false;
	}
	m_err = ET_NOT_ALL_DATA;
	return false;
}

/*
m_name = null terminated string
m_size = m_remain
*/
bool CClient::SendName()
{
	int ret;
	ret = send(m_socket, m_name + (m_size - m_remain), m_remain, 0);
	if(ret == SOCKET_ERROR) {
		m_err = ET_SOCKET_ERROR;
		return false;
	}
	m_remain -= ret;

	if(m_remain) {
		m_err = ET_NOT_ALL_DATA;
		return false;
	}

	m_size = 0;
	return true;
}

bool CServerClient::Process()
{
/*	char buffer[255];
	*buffer = 0; // debug*/
	string buffer;
	switch(m_state) {
	case CS_NAME:
		if(RecvName()) {
			m_state = CS_WAIT;
			m_err = ET_ATTEMPT;
			return false;
		}
		break;
	case CS_CHALLENGE:
		if(RecvChallenge()) {
			m_state = CS_CA;
			return true;
		}
		break;
	case CS_CA:
		/*strcpy(buffer, m_conf->GetEntryValueA(ET_SESSION));
		strcat(buffer, "\\ca.crt");*/
		buffer = m_conf->GetEntryValueA(ET_SESSION);
		buffer += "\\ca.crt";
		if(SendFile(buffer.c_str())) {
			m_state = CS_CSR;
			return true;
		}
		break;
	case CS_CSR:
		/*strcpy(buffer, m_conf->GetEntryValueA(ET_SESSION));
		strcat(buffer, "\\");
		strcat(buffer, m_name);
		strcat(buffer, ".csr");*/
		buffer = m_conf->GetEntryValueA(ET_SESSION);
		buffer += "\\";
		buffer += m_name;
		buffer += ".csr";
		if(RecvFile(buffer.c_str())) {
			if(!m_SignCSR(m_conf, m_name)) {
				m_err = ET_SIGN_ERROR;
			} else {
				m_state = CS_CRT;
				return true;
			}
		}
		break;
	case CS_CRT:
		/*strcpy(buffer, m_conf->GetEntryValueA(ET_SESSION));
		strcat(buffer, "\\");
		strcat(buffer, m_name);
		strcat(buffer, ".crt");*/
		buffer = m_conf->GetEntryValueA(ET_SESSION);
		buffer += "\\";
		buffer += m_name;
		buffer += ".crt";
		if(SendFile(buffer.c_str())) {
			m_state = CS_SHUTDOWN;
			m_err = ET_COMPLETED;
		}
		break;
	case CS_WAIT:
	case CS_SHUTDOWN:
		return false;
	}
/*			m_state = CS_COMPLETE;
			shutdown(m_socket, 1);
			return true;
		}
		break;
	case CS_COMPLETE:
		if(recv(m_socket, buffer, sizeof(buffer), 0) == 0)
			m_err = ET_COMPLETED;
		else
			m_err = ET_SOCKET_ERROR;
		return false;
	}*/

	#ifdef MYDEBUG
		if(m_name)
			cout << m_name << endl;
		else
			cout << "unnamed" << endl;
		cout << buffer << " - ";
		switch(m_err) {
		case ET_FILE_ERROR:
			cout << "file_error" << endl;
			break;
		case ET_SOCKET_ERROR:
			cout << "socket_error" << endl;
			break;
		case ET_SOCKET_CLOSED:
			cout << "socket_closed" << endl;
			break;
		case ET_SIGN_ERROR:
			cout << "sign_error" << endl;
			break;
		case ET_CRYPT_ERROR:
			cout << "crypt_error" << endl;
			break;
		case ET_WRONG_PASSWORD:
			cout << "wrong_password" << endl;
			break;
		case ET_NOT_ALL_DATA:
			cout << "not_all_data" << endl;
			break;
		case ET_NAME_TOO_LONG:
			cout << "name_too_long" << endl;
			break;
		case ET_TIMEOUT:
			cout << "timeout" << endl;
			break;
		case ET_COMPLETED:
			cout << "completed" << endl;
			break;
		}
	#endif

	return false;
}
