/*
nzev: client.h
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#ifndef CLIENT_H
	#define CLIENT_H

#include "peer.h"
#include "crypt.h"
#include "config.h"

namespace Exchanger {

enum ERROR_TYPE {
	ET_FILE_ERROR,
	ET_SOCKET_ERROR,
	ET_SOCKET_CLOSED,
	ET_SIGN_ERROR,
	ET_CRYPT_ERROR,
	ET_WRONG_PASSWORD,
	ET_NOT_ALL_DATA,
	ET_NAME_TOO_LONG,
	ET_TIMEOUT,
	ET_COMPLETED,
	ET_ATTEMPT,
	ET_CLOSED
};

class CClientPeer {
protected:
	static const unsigned int CLIENT_MAX_NAME = 255;
	static const char CLIENT_CHALLENGE[];
	SOCKET m_socket;
	sockaddr_in m_addr;

	CCrypto *m_crypt;
	char *m_buffer, *m_pass, *m_name, *m_ntwrk;
	unsigned int m_size, m_remain;
	ERROR_TYPE m_err;

	enum CLIENTSTATE_TYPE {
		CS_NAME,
		CS_CHALLENGE,
		CS_CA,
		CS_CSR,
		CS_CRT,
		CS_SHUTDOWN,
		CS_WAIT
	} m_state;
public:
	CClientPeer(CCrypto *_crypt): m_buffer(NULL), m_socket(0), m_crypt(_crypt), m_pass(NULL), m_name(NULL), m_ntwrk(NULL) {}
	CClientPeer(SOCKET _socket, const sockaddr_in &_addr, CCrypto *_crypt, const char *_ntwrk);
	virtual ~CClientPeer();

	bool SendFile(const char *_file);
	bool RecvFile(const char *_file);
	bool SendData(const char *_data);
	bool RecvData();
	bool SendChallenge();
	bool RecvChallenge();
	bool Shutdown();
	void SetPasswd(const char *_pass);
	void SetNetwork(const char *_ntwrk);

	inline sockaddr_in GetAddress() const { return m_addr; }
	inline ERROR_TYPE GetLastError() const { return m_err; }
};

class CClient: public CClientPeer, public CPeer {
private:
	CCrypto m_cprov;
	bool SendName();
public:
	CClient(const CConfig *_conf): CPeer(), m_cprov(_conf), CClientPeer(&m_cprov) {}
	virtual ~CClient() {}
	bool Connect(const char *_addr, unsigned short _port);
	bool Exchange(const char *_pass,  const char *_name, const char *_ntwrk);
	bool Process();
};

class CServerClient: public CClientPeer {
private:
	const CConfig *m_conf;

	bool (* m_SignCSR)(const CConfig *, const char *);
	bool RecvName();
public:
	CServerClient(SOCKET _socket, const sockaddr_in _addr, CCrypto *_cprov, const CConfig *_conf, bool (* _SignCSR)(const CConfig *, const char *));
	virtual ~CServerClient() { }

	void fdadd(fd_set *_read, fd_set *_write);
	bool fdisset(fd_set *_set) { return (0 != FD_ISSET(m_socket, _set)); }
	bool Process();
	const char *GetName() { return m_name; }

	void Accept() { m_state = CS_CHALLENGE; }
	void Decline() { m_state = CS_SHUTDOWN; m_err = ET_CLOSED; }
};

}

#endif
