/*
nzev: config.cpp
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#include "config.h"
#include <tchar.h>

bool WtoA(char *_dst, const wchar_t *_src, size_t _size)
{
	BOOL defch;
	int size;
	size = WideCharToMultiByte(CP_ACP, 0, _src, -1, NULL, 0, NULL, &defch);
	if(!size || size > (int)_size || defch)
		return false;
	if(WideCharToMultiByte(CP_ACP, 0, _src, -1, _dst, size, NULL, NULL))
		return true;
	return false;
}

bool AtoW(wchar_t *_dst, const char *_src, size_t _size)
{
	int size;
	size = MultiByteToWideChar(CP_ACP, 0, _src, -1, NULL, 0);
	if(!size || size > (int)_size)
		return false;
	if(MultiByteToWideChar(CP_ACP, 0, _src, -1, _dst, size))
		return true;
	return false;
}

CConfig::CConfig()
{
	for(int i = 0; i < ET_MAXENTRY; i++) {
		memcpy(m_confs[i].valA, default_confs[i].valA, strlen(default_confs[i].valA) + 1);
		memcpy(m_confs[i].valW, default_confs[i].valW, (wcslen(default_confs[i].valW) + 1) * sizeof(wchar_t));
		memcpy(m_shadow[i].valA, default_confs[i].valA, strlen(default_confs[i].valA) + 1);
		memcpy(m_shadow[i].valW, default_confs[i].valW, (wcslen(default_confs[i].valW) + 1) * sizeof(wchar_t));
		m_diff[i] = false;
	}
	InitPath();
	m_flags = 0;
}

/*
zkopruje novou hodnotu do pslunho parametru konfigurace
_et - parametr konfigurace
_value - nov hodnota parametru
_count - velikost hodnoty ve znacch bez ukonovac 0
*/
bool CConfig::SetEntryValue(ENTRY_TYPE _et, LPCTSTR _value, size_t _count)
{
	#ifdef UNICODE
		if(!wcsncmp(_value, m_confs[_et].valW, _count))
			return true;
		if(!WtoA(m_confs[_et].valA, _value, _count))
			return false;
		memcpy(m_confs[_et].valW, _value, _count * sizeof(wchar_t));
		m_confs[_et].valW[_count] = 0;
	#elif
		if(!strncmp(_value, m_confs[_et].valA, _count))
			return true;
		if(!AtoW(m_confs[_et].valW, _value, _count))
			return false;
		memcpy(m_confs[_et].valA, _value, _count);
		m_confs[_et].valA[_count] = 0;
	#endif
	m_diff[_et] = true;
	m_flags |=	(keys[_et].flags & CK_RENEW) |
				(((m_flags & CK_SERVER) << 3) & keys[_et].flags) |
				(((m_flags & CK_SERVER) << 5) & keys[_et].flags);
	return true;
}

void CConfig::Commit()
{
	for(int i = 0; i < ET_MAXENTRY; i++) {
		if(m_diff[i]) {
			memcpy(m_shadow[i].valA, m_confs[i].valA, strlen(m_confs[i].valA) + 1);
			memcpy(m_shadow[i].valW, m_confs[i].valW, (wcslen(m_confs[i].valW) + 1) * sizeof(wchar_t));
			m_diff[i] = false;
		}
	}
	m_backup = m_flags;
}

void CConfig::Rollback()
{
	for(int i = 0; i < ET_MAXENTRY; i++) {
		if(m_diff[i]) {
			memcpy(m_confs[i].valA, m_shadow[i].valA, strlen(m_shadow[i].valA) + 1);
			memcpy(m_confs[i].valW, m_shadow[i].valW, (wcslen(m_shadow[i].valW) + 1) * sizeof(wchar_t));
			m_diff[i] = false;
		}
	}
	m_flags = m_backup;
}

const TCHAR CONFIG_FILE[] = TEXT("\\config.txt");
const wchar_t CONFIG_ENDLINE[] = L"\r\n";
const wchar_t CONFIG_DELIMITER[] = L"=";
const wchar_t CONFIG_SERVER_FLAG[] = L"server";
const wchar_t CONFIG_CLIENT_FLAG[] = L"client";

bool CConfig::LoadConfig()
{
	HANDLE file;
	TCHAR name[MAX_VALUE_LENGTH + sizeof(CONFIG_FILE) / sizeof(TCHAR)];
	_tcsncpy(name, GetEntryValue(ET_SESSION), MAX_VALUE_LENGTH);
	_tcsncat(name, CONFIG_FILE, sizeof(CONFIG_FILE) / sizeof(TCHAR));
	file = CreateFile(	name,
						GENERIC_READ,
						FILE_SHARE_READ,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL);
	if(file == INVALID_HANDLE_VALUE)
		return false;
	const size_t BUFFER_SIZE = MAX_VALUE_LENGTH + 50;
	wchar_t buf[BUFFER_SIZE], *str1, *str2, c = 0;
	DWORD dwRead = 0;
	size_t size, buflen = 0;
	int i;
	bool ret = true;

	while(1) {
		buf[buflen] = c;
		ReadFile(file, (char *)(buf + buflen) + dwRead, sizeof(wchar_t) * (BUFFER_SIZE - 1 - (DWORD)buflen), &dwRead, NULL);
		if(!dwRead && !buflen)
			break;
		buflen += dwRead / sizeof(wchar_t);
		dwRead %= sizeof(wchar_t);
		c = buf[buflen];
		buf[buflen] = 0;

		str1 = wcsstr(buf, CONFIG_DELIMITER);
		str2 = wcsstr(buf, CONFIG_ENDLINE);
		if(!str1 || !str2) {
			// lajna neni kompletni
			ret = false;
			break;
		}
		if((str1 - buf) == 0 || (str2 - str1) == (sizeof(CONFIG_DELIMITER) / sizeof(wchar_t) - 1)) {
			// mezi oddelovaci nic neni
			ret = false;
			break;
		}
		for(i = 0; i < ET_MAXENTRY; i++) {
			size = str1 - buf;
			if(!wcsncmp(buf, keys[i].key, size))
				break;
		}
		if(i != ET_MAXENTRY) {
			// ulozenni dat
			str1 += sizeof(CONFIG_DELIMITER) / sizeof(wchar_t) - 1;
			size = str2 - str1;
			str1[size++] = 0;
			if(!WtoA(m_confs[i].valA, str1, size)) {
				ret = false;
				break;
			}
			memcpy(m_confs[i].valW, str1, size * sizeof(wchar_t));
			m_diff[i] = true;
			m_flags |= keys[i].flags & (CK_REINIT | CK_RESTART);
			if(i == ET_TYPE) {
				if(!wcsncmp(str1, CONFIG_SERVER_FLAG, size))
					m_flags = CK_SERVER;
				else if(!wcsncmp(str1, CONFIG_CLIENT_FLAG, size))
					m_flags = CK_CLIENT;
				else {
					ret = false;
					break;
				}
			}
			size = (str2 + (sizeof(CONFIG_ENDLINE) / sizeof(wchar_t) - 1)) - buf;
			buflen -= size;
			memmove(buf, buf + size, buflen * sizeof(wchar_t));
		} else {
			ret = false;
			break;
		}
	}
	CloseHandle(file);
	m_flags &= ~(((m_flags & CK_CLIENT) << 2) | ((m_flags & CK_CLIENT) << 4));
	return ret;
}

bool CConfig::SaveConfig()
{
	HANDLE file;
	TCHAR name[MAX_VALUE_LENGTH + sizeof(CONFIG_FILE) / sizeof(TCHAR)];
	_tcsncpy(name, GetEntryValue(ET_SESSION), MAX_VALUE_LENGTH);
	_tcsncat(name, CONFIG_FILE, sizeof(CONFIG_FILE) / sizeof(TCHAR));
	file = CreateFile(	name,
						GENERIC_WRITE,
						0,
						NULL,
						CREATE_ALWAYS,
						FILE_ATTRIBUTE_NORMAL,
						NULL);
	if(file == INVALID_HANDLE_VALUE)
		return false;
	int i;
	DWORD dwWrite, dwWritten;
	if(m_flags & CK_SERVER)
		wcsncpy(m_confs[ET_TYPE].valW, CONFIG_SERVER_FLAG, MAX_VALUE_LENGTH);
	else
		wcsncpy(m_confs[ET_TYPE].valW, CONFIG_CLIENT_FLAG, MAX_VALUE_LENGTH);

	for(i = 0; i < ET_MAXENTRY; i++) {
		if(((m_flags & keys[i].flags) & (CK_CLIENT | CK_SERVER)) | (keys[i].flags & CK_CREATE) | (i == ET_TYPE)) {
			dwWrite = (DWORD)wcslen(keys[i].key) * sizeof(wchar_t);
			WriteFile(file, keys[i].key, dwWrite, &dwWritten, NULL);
			WriteFile(file, CONFIG_DELIMITER, sizeof(CONFIG_DELIMITER) - sizeof(wchar_t), &dwWritten, NULL);
			dwWrite = (DWORD)wcslen(m_confs[i].valW) * sizeof(wchar_t);
			WriteFile(file, m_confs[i].valW, dwWrite, &dwWritten, NULL);
			WriteFile(file, CONFIG_ENDLINE, sizeof(CONFIG_ENDLINE) - sizeof(wchar_t), &dwWritten, NULL);
		}
	}
	CloseHandle(file);
	return true;
}

bool CConfig::InitPath()
{
	static const TCHAR szOvpn[] = TEXT("SOFTWARE\\OpenVPN");
	HKEY hkey;

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, szOvpn, 0, KEY_READ, &hkey) != ERROR_SUCCESS) {
		return false;
	}

	TCHAR buffer[MAX_VALUE_LENGTH];
	DWORD size = sizeof(buffer);

	bool ret;
	ret = (RegQueryValueEx(hkey, NULL, NULL, NULL, (LPBYTE)buffer, &size) == ERROR_SUCCESS);
	RegCloseKey(hkey);

	if(ret && SetEntryValue(ET_PATH, buffer, size / sizeof(TCHAR))) {
		Commit();
		return true;
	}
	return false;
}
