/*
nzev: config.h
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#ifndef CONFIG_H
	#define CONFIG_H

#include <windows.h>

enum ENTRY_TYPE {
	ET_SESSION,
	ET_TAPNAME,
	ET_LOOPBACKNAME,
	ET_BRIDGENAME,
	ET_BRIDGEADDRESS,
	ET_BRIDGEMASK,
	ET_NICNAME,
	ET_SERVERADDRESS,
	ET_SERVERPORT,
	ET_OUTPORT,
	ET_PROTOCOL,
	ET_IPRANGE1,
	ET_IPRANGE2,
	ET_KEYSIZE,
	ET_EXCHANGERADDRESS,
	ET_EXCHANGERPORT,
	ET_CLIENTNAME,
	ET_CLIENTPASS,
	ET_COUNTRY,
	ET_CITY,
	ET_PATH,
	ET_TYPE,
	ET_MAXENTRY
};

const unsigned int CK_SERVER = 0x1;
const unsigned int CK_CLIENT = 0x2;
const unsigned int CK_RENEW = 0x4;
const unsigned int CK_REINIT = 0x8;
const unsigned int CK_CREATE = 0x10;
const unsigned int CK_RESTART = 0x20;

struct ConfigKey {
	wchar_t *key;
	TCHAR *label;
	unsigned int flags;
};

static const ConfigKey keys[ET_MAXENTRY] = {
	{ L"session", TEXT("s"), CK_CREATE },
	{ L"tapname", TEXT("tap adaptr"), CK_SERVER | CK_CLIENT },
	{ L"loopbackname", TEXT("loopback adaptr"), CK_SERVER },
	{ L"bridgename", TEXT("sov most"), CK_SERVER | CK_REINIT },
	{ L"bridgeaddress", TEXT("adresa sovho mostu"), CK_SERVER | CK_REINIT },
	{ L"bridgemask", TEXT("maska sovho mostu"), CK_SERVER | CK_REINIT },
	{ L"nicname", TEXT("sov adaptr"), CK_SERVER | CK_REINIT },
	{ L"serveraddress", TEXT("adresa VPN serveru"), CK_CLIENT },
	{ L"serverport", TEXT("port VPN serveru"), CK_SERVER | CK_CLIENT | CK_REINIT },
	{ L"outport", TEXT("vnj port pro NAT"), CK_SERVER | CK_REINIT },
	{ L"protocol", TEXT("protokol VPN pipojen"), CK_SERVER | CK_CLIENT | CK_REINIT },
	{ L"iprange1", TEXT("zatek rozsahu adres"), CK_SERVER },
	{ L"iprange2", TEXT("konec rozsahu adres"), CK_SERVER },
	{ L"keysize", TEXT("velikost kle"), CK_SERVER | CK_CLIENT | CK_RENEW },
	{ L"exchangeraddress", TEXT("adresa exchange serveru"), CK_CLIENT },
	{ L"exchangerport", TEXT("port exchange serveru"), CK_SERVER | CK_CLIENT | CK_RESTART },
	{ L"clientname", TEXT("jmno klienta"), CK_CLIENT | CK_RENEW },
	{ L"clientpass", TEXT("heslo"), CK_SERVER | CK_CLIENT | CK_RESTART },
	{ L"country", TEXT("kd zem"), CK_SERVER | CK_CLIENT | CK_RENEW },
	{ L"city", TEXT("msto"), CK_SERVER | CK_CLIENT | CK_RENEW },
	{ L"path", NULL, 0 },
	{ L"type", NULL, 0 }
};

static const size_t MAX_VALUE = 255;

struct ConfigEntry {
	wchar_t valW[MAX_VALUE];
	char valA[MAX_VALUE];
};

static const ConfigEntry default_confs[ET_MAXENTRY] = {
	{ L"default", "default" },
	{ L"TAP", "TAP" },
	{ L"Virtual", "Virtual" },
	{ L"bridge", "bridge" },
	{ L"192.168.1.1", "192.168.1.1" },
	{ L"255.255.255.0", "255.255.255.0" },
	{ L"LAN", "LAN" },
	{ L"192.168.1.1", "192.168.1.1" },
	{ L"1194", "1194" },
	{ L"1194", "1194" },
	{ L"udp", "udp" },
	{ L"192.168.1.2", "192.168.1.2" },
	{ L"192.168.1.100", "192.168.1.100" },
	{ L"1024", "1024" },
	{ L"localhost", "localhost" },
	{ L"5000", "5000" },
	{ L"pepa", "pepa" },
	{ L"heslo", "heslo" },
	{ L"CZ", "CZ" },
	{ L"Liberec", "Liberec" },
	{ L"C:\\Program Files\\OpenVPN", "C:\\Program Files\\OpenVPN" },
	{ L"client", "client" }
};

class CConfig {
private:
	ConfigEntry m_confs[ET_MAXENTRY];
	ConfigEntry m_shadow[ET_MAXENTRY];
	bool m_diff[ET_MAXENTRY];
	unsigned int m_backup;

	bool InitPath();
public:
	unsigned int m_flags;
	CConfig();
	static const size_t MAX_VALUE_LENGTH = MAX_VALUE;

	const TCHAR *GetEntryLabel(ENTRY_TYPE _et) const { return keys[_et].label; }
	unsigned int GetEntryFlags(ENTRY_TYPE _et) const { return keys[_et].flags; }
	const TCHAR *GetEntryValue(ENTRY_TYPE _et) const {
		#ifdef UNICODE
			return GetEntryValueW(_et);
		#elif
			return GetEntryValueA(_et);
		#endif
	}
	const wchar_t *GetEntryValueW(ENTRY_TYPE _et) const { return m_confs[_et].valW; }
	const char *GetEntryValueA(ENTRY_TYPE _et) const { return m_confs[_et].valA; }

	bool SetEntryValue(ENTRY_TYPE _et, LPCTSTR _value, size_t _count);
	bool LoadConfig();
	bool SaveConfig();
	void Commit();
	void Rollback();
};

#endif
