/*
nzev: exchanger.cpp
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#include <tchar.h>
#include <sstream>
#include "exchanger.h"
#include "interface.h"
#include "manager.h"

using namespace std;
using namespace Exchanger;

static HWND hwnd;

CExchanger::~CExchanger()
{
	if(m_peer)
		delete m_peer;
}

int Exchanger::AtoT(TCHAR * &dst, const char *src)
{
	#ifdef UNICODE
		int size;
		wchar_t *buf;
		size = MultiByteToWideChar(CP_ACP, 0, src, -1, NULL, 0);
		if(!size)
			return 0;
		buf = new wchar_t[size];
		if(!MultiByteToWideChar(CP_ACP, 0, src, -1, buf, size)) {
			delete [] buf;
			return 0;
		}
		dst = buf;
		return 1;
	#else
		dst = src;
		return -1;
	#endif
}

bool Exchanger::SignCSR(const CConfig *_conf, const char *_client)
{
	TCHAR *client;
	if(!AtoT(client, _client))
		return false;
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\signcsr.bat"),
		_conf->GetEntryValue(ET_PATH),
		_conf->GetEntryValue(ET_SESSION),
		client,
		_conf->GetEntryValue(ET_KEYSIZE),
		_conf->GetEntryValue(ET_COUNTRY),
		_conf->GetEntryValue(ET_CITY)
	};
	return RunScript<true>(7, argv);
}

void Exchanger::NotifyCallback(NOTIFY_TYPE _event, const char *_name, const sockaddr_in *_addr)
{
	CLIENTINFO_TYPE *cit = new CLIENTINFO_TYPE;
	size_t size = strlen(_name) + 1;
	cit->name = new char[size];
	memcpy(cit->name, _name, size);

	ostringstream os;
	os	<< (int)_addr->sin_addr.S_un.S_un_b.s_b1 << '.'
		<< (int)_addr->sin_addr.S_un.S_un_b.s_b2 << '.'
		<< (int)_addr->sin_addr.S_un.S_un_b.s_b3 << '.'
		<< (int)_addr->sin_addr.S_un.S_un_b.s_b4 << ':'
		<< _addr->sin_port;
	os.flush();
	string &str = os.str();

	size = strlen(str.c_str()) + 1;
	cit->addr = new char[size];
	memcpy(cit->addr, str.c_str(), size);
	PostMessage(hwnd, WM_EXCHANGER_NOTIFY, (WPARAM)_event, (LPARAM)cit);
}

bool CExchanger::StartServer()
{
	hwnd = m_hwnd;
	m_peer = new CServer(m_conf);
	CServer *server = (CServer *)m_peer;
	unsigned int port = _ttoi(m_conf->GetEntryValue(ET_EXCHANGERPORT));
	if(port > 0 && port <= 0xFFFF) {
		if(server->Init((unsigned short)port, SignCSR, NotifyCallback))
			return true;
	}
	delete server;
	m_peer = NULL;
	return false;
}

bool CExchanger::StartClient()
{
	m_peer = new CClient(m_conf);
	CClient *client = (CClient *)m_peer;
	unsigned int port = _ttoi(m_conf->GetEntryValue(ET_EXCHANGERPORT));
	bool ret = true;
	if(port == 0 || port > 0xFFFF)
		ret = false;
	if(ret) {
		ret = client->Connect(m_conf->GetEntryValueA(ET_EXCHANGERADDRESS), (unsigned short)port);
	}
	if(ret) {
		ret = client->Exchange(m_conf->GetEntryValueA(ET_CLIENTPASS), m_conf->GetEntryValueA(ET_CLIENTNAME), m_conf->GetEntryValueA(ET_SESSION));
	}
	delete client;
	m_peer = NULL;
	return ret;
}

void CExchanger::AcceptClient(const char *_name, const char *_pass)
{
	MESSAGE_TYPE msg;
	msg.accept = true;
	size_t size = strlen(_name) + 1;
	msg.name = new char[size];
	memcpy(msg.name, _name, size);
	size = strlen(_pass) + 1;
	msg.passwd = new char[size];
	memcpy(msg.passwd, _pass, size);

	((CServer *)m_peer)->PostMessage(&msg);
}

void CExchanger::DeclineClient(const char *_name)
{
	MESSAGE_TYPE msg;
	msg.accept = false;
	size_t size = strlen(_name) + 1;
	msg.name = new char[size];
	memcpy(msg.name, _name, size);
	msg.passwd = NULL;

	((CServer *)m_peer)->PostMessage(&msg);
}
