/*
nzev: interface.cpp
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#define _WIN32_DCOM
#include <comdef.h>
#include <comutil.h>
#include <wbemidl.h>
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include <set>
#include <vector>
#include "interface.h"
#include <iostream>
#include <iomanip>

using namespace std;

bool ShellExecuteWait(DWORD& dwRet, int nShow, TCHAR *tcExe, TCHAR* tcWorkingDir = NULL, TCHAR* tcArgs = NULL)
{
	SHELLEXECUTEINFO sexi;
	ZeroMemory(&sexi, sizeof(SHELLEXECUTEINFO));
	sexi.cbSize = sizeof(SHELLEXECUTEINFO);
	sexi.fMask = SEE_MASK_NOCLOSEPROCESS;
	sexi.lpDirectory = tcWorkingDir;
	sexi.lpFile = tcExe;
	sexi.lpParameters = tcArgs;
	sexi.nShow = nShow;

	if(ShellExecuteEx(&sexi)) {
		DWORD dwSignaled = WaitForSingleObject(sexi.hProcess, INFINITE);
		if(dwSignaled == WAIT_OBJECT_0) {
			if(GetExitCodeProcess(sexi.hProcess, &dwRet))
				return true;
		}
	}
	dwRet = GetLastError();
	return false;
}

void ShellExecuteAsync(TCHAR *tcExe, TCHAR* tcWorkingDir = NULL, TCHAR* tcArgs = NULL)
{
	SHELLEXECUTEINFO sexi;
	ZeroMemory(&sexi, sizeof(SHELLEXECUTEINFO));
	sexi.cbSize = sizeof(SHELLEXECUTEINFO);
	sexi.lpDirectory = tcWorkingDir;
	sexi.lpFile = tcExe;
	sexi.lpParameters = tcArgs;
	sexi.nShow = SW_SHOWNORMAL;

	ShellExecuteEx(&sexi);
}

TCHAR *WrapCmdline(int length, const TCHAR * const argv[])
{
	size_t size = 0;
	for(int i = 0; i < length; i++) {
		size += _tcslen(argv[i]) + 3;
	}
	size -= 2;
	TCHAR *line = new TCHAR[size];
	_tcscpy(line, argv[0]); // nen poteba bezpen verze
	for(int i = 1; i < length; i++) {
		_tcscat(line, TEXT(" \""));
		_tcscat(line, argv[i]);
		_tcscat(line, TEXT("\""));
	}
	return line;
}
/*
bool InstallNic(INTERFACE_TYPE type)
{
	bool ret = false;
	switch(type) {
	case IT_LOOPBACK:
		ret = RunScript<true>(TEXT("/K scripts\\install_nic.bat \"%WINDIR%\\inf\\netloop.inf\" \"*msloop\""));
		break;
	case IT_BRIDGE:
		ret =RunScript<true>(TEXT("/K scripts\\install_nic.bat \"%WINDIR%\\inf\\netbrdgm.inf\" \"ms_bridgemp\""));
		break;
	case IT_TAP:
		ret = RunScript<true>(TEXT("/K scripts\\install_nic.bat \"%PROGRAMFILES%\\OpenVPN\\driver\\OemWin2k.inf\" \"tap0801\""));
		break;
	};
	return ret;
}
*/
bool RenameNic(const TCHAR *nic_name, const TCHAR *new_name)
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\rename_nic.bat"),
		nic_name,
		new_name,
	};
	return RunScript<true>(3, argv);
}

bool InstallLoop(const CConfig *_conf)
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\install_nic.bat"),
		_conf->GetEntryValue(ET_PATH),
		TEXT("%WINDIR%\\inf\\netloop.inf"),
		TEXT("*msloop")
	};
	return RunScript<true>(4, argv);
}

bool InstallBridge(const CConfig *_conf)
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\install_nic.bat"),
		_conf->GetEntryValue(ET_PATH),
		TEXT("%WINDIR%\\inf\\netbrdgm.inf"),
		TEXT("ms_bridgemp")
	};
	return RunScript<true>(4, argv);
}

bool InstallTAP(const CConfig *_conf)
{
	const TCHAR drv[] = TEXT("\\driver\\OemWin2k.inf");
	TCHAR buffer[CConfig::MAX_VALUE_LENGTH + sizeof(drv) / sizeof(TCHAR)];
	_tcsncpy(buffer, _conf->GetEntryValue(ET_PATH), CConfig::MAX_VALUE_LENGTH);
	_tcsncat(buffer, drv, sizeof(drv) / sizeof(TCHAR));
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\install_nic.bat"),
		_conf->GetEntryValue(ET_PATH),
		buffer,
		TEXT("tap0801")
	};
	return RunScript<true>(4, argv);
}

template<bool unitilize> class CReleaser {
	IUnknown *m_pCOM;
public:
	CReleaser(IUnknown *pCOM) { m_pCOM = pCOM; }
	~CReleaser() { if(unitilize) CoUninitialize(); else m_pCOM->Release(); }
};

TCHAR *AddInterface(const CConfig *_conf, INTERFACE_TYPE _type)
{
	static const BSTR adapter_query = BSTR(TEXT("SELECT * FROM Win32_NetworkAdapter"));
	static const TCHAR *param_name = TEXT("NetConnectionID");

	HRESULT hr;
	hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	if(hr != S_OK)
		return NULL;

	CReleaser<true> r1(NULL);

	hr = CoInitializeSecurity( NULL, -1, NULL, NULL, RPC_C_AUTHN_LEVEL_DEFAULT, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE, NULL);   
	if(hr != S_OK)
		return NULL;

	IWbemLocator *pLoc = 0;

	hr = CoCreateInstance(CLSID_WbemLocator, 0, CLSCTX_INPROC_SERVER, IID_IWbemLocator, (LPVOID *)&pLoc);
	if(hr != S_OK)
		return NULL;

	CReleaser<false> r2(pLoc);

	IWbemServices *pSvc = 0;

	hr = pLoc->ConnectServer(BSTR(TEXT("root\\cimv2")), NULL, NULL, 0, NULL, 0, 0, &pSvc);
	if(hr != S_OK)
		return NULL;

	CReleaser<false> r3(pSvc);

	hr = CoSetProxyBlanket(pSvc, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE, NULL, RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE);
	if(hr != S_OK)
		return NULL;

    IEnumWbemClassObject *pEnum;
    hr = pSvc->ExecQuery(BSTR(TEXT("WQL")), adapter_query, WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, NULL, &pEnum);
	if(hr != S_OK)
		return NULL;

	IWbemClassObject *pclsObj;
    ULONG uReturn = 0;
	VARIANT vtProp;

	set<bstr_t> name_set;

	while(pEnum) {
		hr = pEnum->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);

		if(uReturn == 0)
			break;

        hr = pclsObj->Get(param_name, 0, &vtProp, 0, 0);
		if(vtProp.vt != VT_NULL)
			name_set.insert(vtProp.bstrVal);

        VariantClear(&vtProp);
		pclsObj->Release();
	}
	pEnum->Release();

	bool ret;
	switch(_type) {
	case IT_TAP:
		ret = InstallTAP(_conf);
		break;
	case IT_LOOPBACK:
		ret = InstallLoop(_conf);
		break;
	case IT_BRIDGE:
		ret = InstallBridge(_conf);
		break;
	default:
		ret = false;
	}
	if(!ret)
		return NULL;

	hr = pSvc->ExecQuery(BSTR(TEXT("WQL")), adapter_query, WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, NULL, &pEnum);
	if(hr != S_OK)
		return NULL;

	CReleaser<false> r4(pEnum);

    while(pEnum) {
		hr = pEnum->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);

		if(uReturn == 0)
			break;

        hr = pclsObj->Get(param_name, 0, &vtProp, 0, 0);
		if(vtProp.vt != VT_NULL)
			if(name_set.erase(vtProp.bstrVal) == 0) {
				bstr_t val(vtProp.bstrVal);
				TCHAR *name = new TCHAR[val.length() + 1];
				memcpy(name, (TCHAR *)val, val.length() * sizeof(TCHAR));
				name[val.length()] = 0;
				VariantClear(&vtProp);
				pclsObj->Release();
				return name;
			}
        VariantClear(&vtProp);
		pclsObj->Release();
    }

	return NULL;
}
