/*
nzev: logger.h
autor: Pavel Hasenhrl
datum: 17.5.2009
verze: 1.0
*/

#ifndef LOGGER_H
	#define LOGGER_H

#include <windows.h>
#include <tchar.h>
#include <sstream>
#include <iomanip>

template<typename Elem> class CLogger {
private:
	HWND m_parent;
	HWND m_log;
	size_t m_pos;
	static const int TIME_LENGTH = 20;
	static const int MAX_LINE_LENGTH = 260 + TIME_LENGTH;
	Elem m_buffer[MAX_LINE_LENGTH];
public:
	CLogger(HINSTANCE _hInst, HWND _parent);
	void resize(int _rx, int _ry) { MoveWindow(m_log, 0, 0, _rx, _ry, TRUE); }
	void append(const Elem *_text);
};

template<typename Elem> CLogger<Elem>::CLogger(HINSTANCE _hInst, HWND _parent)
{
	m_log = CreateWindowEx	(	0, TEXT("EDIT"), NULL, WS_CHILD | WS_MAXIMIZE | WS_HSCROLL | WS_VSCROLL | WS_VISIBLE | ES_AUTOVSCROLL | ES_MULTILINE | ES_READONLY,
							0, 0, 0, 0, _parent, NULL, _hInst, NULL
						);
	m_pos = 0;
}

template<typename Elem> void CLogger<Elem>::append(const Elem *_text)
{
	SYSTEMTIME time;
	GetLocalTime(&time);

	std::basic_ostringstream<Elem> os;
	os.fill('0');
	os	<< std::setw(2) << time.wHour << ':'
		<< std::setw(2) << time.wMinute << ':'
		<< std::setw(2) << time.wSecond;
	os.fill(' ');
	os << std::setw(TIME_LENGTH - 7) << ' ';
	os.flush();

	const basic_string<Elem> str = os.str();

	Elem *buf = m_buffer;
	size_t size;
	memcpy(buf, str.c_str(), TIME_LENGTH * sizeof(Elem));
	buf += TIME_LENGTH;
	size = _tcslen(_text);
	memcpy(buf, _text, size * sizeof(Elem));
	buf += size;
	const Elem ENDLINE[] = TEXT("\r\n");
	memcpy(buf, ENDLINE, sizeof(ENDLINE));
	buf += sizeof(ENDLINE) / sizeof(Elem) - 1;

	SendMessage(m_log, EM_SETSEL, m_pos, m_pos);
	SendMessage(m_log, EM_REPLACESEL, FALSE, (LPARAM)m_buffer);
	m_pos += (buf - m_buffer);
}

#endif
