/*
nzev: main.cpp
autor: Pavel Hasenhrl
datum: 17.5.2009
verze: 1.0
*/

#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <sstream>
#include "resource.h"
#include "exchanger.h"
#include "manager.h"
#include "logger.h"

using namespace std;
using namespace Exchanger;

HINSTANCE g_hInst;
CConfig *g_config;
CExchanger *g_exchanger;
CManager *g_manager;
CLogger<TCHAR> *g_logger;
const size_t MAX_FILEPATH = 255; // maximln dlka cesty k souboru
const size_t STRING_BUFFER_SIZE = 255;
TCHAR g_szBuffer[STRING_BUFFER_SIZE];

HWND InitWindow(HINSTANCE _hInst);
LRESULT CALLBACK MainWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK DlgSettingProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

void ExchangeComplete(CLIENTINFO_TYPE *_cit);
void ExchangeAttempt(HWND _hwnd, CLIENTINFO_TYPE *_cit);
bool ChangeConfig(HWND _hwnd, CConfig *_conf);
void PrintChanges(CConfig *_conf);
bool OpenConfig(HWND _hwnd, CConfig *_conf);

const TCHAR CLASS_NAME[] = TEXT("OPENMACHI");
const TCHAR WINDOW_TITLE[] = TEXT("openmachi");

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// vytvoen okna
	HWND hwnd;
	hwnd = InitWindow(hInstance);
	if(!hwnd)
		return -1;

	INITCOMMONCONTROLSEX icc;
	icc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icc.dwICC = ICC_BAR_CLASSES;
	InitCommonControlsEx(&icc);

	// nastaven globlnch promnnch
	g_hInst = hInstance;
	g_config = new CConfig();
	g_exchanger = new CExchanger(g_config);
	g_exchanger->Init(hwnd);
	g_manager = new CManager(g_config);
	g_logger = new CLogger<TCHAR>(hInstance, hwnd);

	ShowWindow(hwnd, SW_SHOWNORMAL);

	// smyka zprv
	MSG message;
	while(GetMessage(&message, NULL, 0, 0)) {
		TranslateMessage(&message);
		DispatchMessage(&message);
	}

	delete g_exchanger;
	delete g_manager;
	delete g_config;

	return (int)message.wParam; // vrac tzv. exit code
}

/*
procedura hlavnho okna
*/
LRESULT CALLBACK MainWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case ID_CONFIG_NEW_SERVER:
			g_config->m_flags = CK_SERVER;
			if(!DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDR_SETTINGS), hwnd, DlgSettingProc, CK_CREATE))
				break;
			if(g_manager->NewServer()) {
				EnableMenuItem(GetMenu(hwnd), 1, MF_BYPOSITION | MF_ENABLED);
				EnableMenuItem(GetMenu(hwnd), 2, MF_BYPOSITION | MF_GRAYED);
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SAVE, MF_ENABLED);
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SETUP, MF_ENABLED);
				LoadString(g_hInst, IDS_NETWORK_CREATED, g_szBuffer, STRING_BUFFER_SIZE);
				PostMessage(hwnd, WM_COMMAND, ID_CONFIG_SAVE, 0);
			} else
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_CONFIG_NEW_CLIENT:
			g_config->m_flags = CK_CLIENT;
			if(!DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDR_SETTINGS), hwnd, DlgSettingProc, CK_CREATE))
				break;
			if(g_manager->NewClient()) {
				EnableMenuItem(GetMenu(hwnd), 1, MF_BYPOSITION | MF_GRAYED);
				EnableMenuItem(GetMenu(hwnd), 2, MF_BYPOSITION | MF_ENABLED);
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SAVE, MF_ENABLED);
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SETUP, MF_ENABLED);
				LoadString(g_hInst, IDS_NETWORK_CREATED, g_szBuffer, STRING_BUFFER_SIZE);
				PostMessage(hwnd, WM_COMMAND, ID_CONFIG_SAVE, 0);
			} else
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_CONFIG_SETUP:
			if(DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDR_SETTINGS), hwnd, DlgSettingProc, 0)) {
				PrintChanges(g_config);
			}
			break;
		case ID_CONFIG_LOAD:
			if(OpenConfig(hwnd, g_config) && g_config->LoadConfig()) {
				LoadString(g_hInst, IDS_CONFIG_LOADED, g_szBuffer, STRING_BUFFER_SIZE);
				g_logger->append(g_szBuffer);
				PrintChanges(g_config);
				//g_config->Commit();
				if(g_config->m_flags & CK_SERVER) {
					EnableMenuItem(GetMenu(hwnd), 2, MF_BYPOSITION | MF_GRAYED);
					EnableMenuItem(GetMenu(hwnd), 1, MF_BYPOSITION | MF_ENABLED);
				} else {
					EnableMenuItem(GetMenu(hwnd), 1, MF_BYPOSITION | MF_GRAYED);
					EnableMenuItem(GetMenu(hwnd), 2, MF_BYPOSITION | MF_ENABLED);
				}
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SAVE, MF_ENABLED);
				EnableMenuItem(GetMenu(hwnd), ID_CONFIG_SETUP, MF_ENABLED);
			} else {
				g_config->Rollback();
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
				g_logger->append(g_szBuffer);
			}
			break;
		case ID_CONFIG_SAVE:
			if(g_config->SaveConfig())
				LoadString(g_hInst, IDS_CONFIG_SAVED, g_szBuffer, STRING_BUFFER_SIZE);
			else
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_SERVER_EXCHANGER_START:
			if(g_exchanger->StartServer()) {
				LoadString(g_hInst, IDS_EXCHANGER_STARTED, g_szBuffer, STRING_BUFFER_SIZE);
				EnableMenuItem(GetMenu(hwnd), ID_SERVER_EXCHANGER_START, MF_GRAYED);
				EnableMenuItem(GetMenu(hwnd), ID_SERVER_EXCHANGER_STOP, MF_ENABLED);
			} else {
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			}
			g_logger->append(g_szBuffer);
			break;
		case ID_SERVER_EXCHANGER_STOP:
			g_exchanger->Stop();
			EnableMenuItem(GetMenu(hwnd), ID_SERVER_EXCHANGER_START, MF_ENABLED);
			EnableMenuItem(GetMenu(hwnd), ID_SERVER_EXCHANGER_STOP, MF_GRAYED);
			LoadString(g_hInst, IDS_EXCHANGER_STOPED, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_SERVER_START:
			g_manager->RunServer();
			break;
		case ID_SERVER_ENABLE:
			if(g_manager->EnableServer())
				LoadString(g_hInst, IDS_SERVER_ENABLED, g_szBuffer, STRING_BUFFER_SIZE);
			else
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_SERVER_DISABLE:
			g_manager->DisableServer();
			LoadString(g_hInst, IDS_SERVER_DISABLED, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_CLIENT_EXCHANGE:
			if(g_exchanger->StartClient())
				LoadString(g_hInst, IDS_EXCHANGE_COMPLETED, g_szBuffer, STRING_BUFFER_SIZE);
			else
				LoadString(g_hInst, IDS_ERROR, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case ID_CLIENT_CONNECT:
			g_manager->RunClient();
			break;
		}
		break;
	case WM_EXCHANGER_NOTIFY:
		{
			CLIENTINFO_TYPE * cit = (CLIENTINFO_TYPE *)lParam;
			switch((NOTIFY_TYPE)wParam) {
			case NT_COMPLETE:
				ExchangeComplete(cit);
				break;
			case NT_ATTEMPT:
				ExchangeAttempt(hwnd, cit);
				break;
			}
			delete [] cit->addr;
			delete [] cit->name;
			delete cit;
		}
		break;
	case WM_SIZE:
		g_logger->resize(LOWORD(lParam), HIWORD(lParam)); 
		break;
/* ukonen aplikace */
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	}
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

/*
Zaregistrovn tdy okna a jeho vytvoen
_hInst - handler modulu (parametr WinMain)
*/
HWND InitWindow(HINSTANCE _hInst)
{
	WNDCLASSEX wc;
	wc.cbSize = sizeof(wc);
	wc.hInstance = _hInst;
	wc.lpfnWndProc = MainWindowProc;
	wc.lpszClassName = CLASS_NAME;
	wc.lpszMenuName = MAKEINTRESOURCE(IDR_MAINMENU);
	wc.hIcon = NULL;//LoadIcon(_hInst, MAKEINTRESOURCE(IDR_MAINICON));
	wc.hIconSm = NULL;
	wc.hbrBackground = (HBRUSH)(COLOR_BTNTEXT + 1);
	wc.hCursor = NULL;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	RegisterClassEx(&wc);

	return CreateWindowEx	(	0, CLASS_NAME, WINDOW_TITLE, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
								CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, _hInst, NULL
							);
}

const int BUTTON_WIDTH = 180;
const int BUTTON_HEIGHT = 20;
const int BUTTON_ALIGN = 10;
const int IDC_SETTINGS_EDIT = 5000;
const int IDC_SETTINGS_BUTTON = 6000;

/*
procedura dialogu s nastavenm
*/
BOOL CALLBACK DlgSettingProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
	case WM_INITDIALOG:
		{
			HWND edit;
			int vpos = BUTTON_ALIGN;
			for(int i = 0; i < ET_MAXENTRY; i++) {
				if(!((g_config->m_flags | lParam) & g_config->GetEntryFlags((ENTRY_TYPE)i)))
					continue;
				CreateWindowEx	(	0, TEXT("STATIC"), g_config->GetEntryLabel((ENTRY_TYPE)i), WS_CHILD | WS_VISIBLE,
									BUTTON_ALIGN, vpos,
									BUTTON_WIDTH, BUTTON_HEIGHT, hwndDlg, NULL,
									g_hInst, NULL
								);
				edit = CreateWindowEx	(	0, TEXT("EDIT"), g_config->GetEntryValue((ENTRY_TYPE)i), SS_SUNKEN | ES_AUTOHSCROLL | WS_CHILD | WS_VISIBLE,
											BUTTON_ALIGN * 2 + BUTTON_WIDTH, vpos,
											BUTTON_WIDTH, BUTTON_HEIGHT, hwndDlg, (HMENU)(IDC_SETTINGS_EDIT + i),
											g_hInst, NULL
										);
				SendMessage(edit, EM_SETLIMITTEXT, g_config->MAX_VALUE_LENGTH - 1, 0);
				if((g_config->GetEntryFlags((ENTRY_TYPE)i) & CK_RENEW) && lParam != CK_CREATE)
					EnableWindow(edit, FALSE);
				switch(i) {
				case ET_TAPNAME:
				case ET_LOOPBACKNAME:
				case ET_BRIDGENAME:
					LoadString(g_hInst, IDS_BUTTON_INSTALL, g_szBuffer, STRING_BUFFER_SIZE);
					CreateWindowEx	(	0, TEXT("BUTTON"), g_szBuffer, WS_CHILD | WS_VISIBLE,
										BUTTON_ALIGN * 3 + 2 * BUTTON_WIDTH, vpos,
										BUTTON_WIDTH, BUTTON_HEIGHT, hwndDlg, (HMENU)(IDC_SETTINGS_BUTTON + i),
										g_hInst, NULL
									);
					break;
				}
				vpos += BUTTON_ALIGN + BUTTON_HEIGHT;
			}
			SetWindowPos(hwndDlg, HWND_TOP, 0, 0, 3 * BUTTON_WIDTH + 4 * BUTTON_ALIGN, 2 * (BUTTON_HEIGHT + BUTTON_ALIGN) + vpos, SWP_NOMOVE | SWP_NOZORDER);
			SetWindowPos(GetDlgItem(hwndDlg, IDC_BUTTON_CANCEL), HWND_TOP, BUTTON_ALIGN, vpos, BUTTON_WIDTH, BUTTON_HEIGHT, SWP_NOZORDER);
			SetWindowPos(GetDlgItem(hwndDlg, IDC_BUTTON_OK), HWND_TOP, 2 * BUTTON_WIDTH + 3 * BUTTON_ALIGN, vpos, BUTTON_WIDTH, BUTTON_HEIGHT, SWP_NOZORDER);
		}
		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case IDC_SETTINGS_BUTTON + ET_TAPNAME:
		case IDC_SETTINGS_BUTTON + ET_LOOPBACKNAME:
		case IDC_SETTINGS_BUTTON + ET_BRIDGENAME:
			if(!ChangeConfig(hwndDlg, g_config))
				break;
			if(g_manager->Install((ENTRY_TYPE)(LOWORD(wParam) - IDC_SETTINGS_BUTTON)))
				LoadString(g_hInst, IDS_INSTALL_SUCCEEDED, g_szBuffer, STRING_BUFFER_SIZE);
			else
				LoadString(g_hInst, IDS_INSTALL_FAILED, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			break;
		case IDC_BUTTON_OK: // potrvrzen dialogu
			if(ChangeConfig(hwndDlg, g_config))
				EndDialog(hwndDlg, TRUE);
			break;
		case IDC_BUTTON_CANCEL: // ukonen dialogu
			EndDialog(hwndDlg, FALSE); // dn zmny se neulo
			break;
		default:
			return FALSE;
		}
		return TRUE;
	}
	return FALSE;
}

/*
notifikuje uivatele serveru o spn distribuc certifikt
_cit - informace o klientovi, kterho se to tk
*/
void ExchangeComplete(CLIENTINFO_TYPE *_cit)
{
	basic_ostringstream<TCHAR> os;

	LoadString(g_hInst, IDS_EXCHANGE_COMPLETED, g_szBuffer, STRING_BUFFER_SIZE);
	os << _cit->name << ' ' << _cit->addr << ' ' << g_szBuffer;
	os.flush();
	g_logger->append(os.str().c_str());
}

/*
notifikuje uivatele serveru o novm pokusu o distribuci certifikt
zobraz dialog s monostmi pijmout nebo odmtnout klienta
_hwnd - handler hlavnho okna
_cit - informace o novm klientovi
*/
void ExchangeAttempt(HWND _hwnd, CLIENTINFO_TYPE *_cit)
{
	basic_ostringstream<TCHAR> os;
	os << _cit->name << ' ' << _cit->addr;
	os.flush();

	LoadString(g_hInst, IDS_EXCHANGE_NEW, g_szBuffer, STRING_BUFFER_SIZE);
	if(MessageBoxEx(_hwnd, os.str().c_str(), g_szBuffer, MB_YESNO, 0) == IDYES) {
		g_exchanger->AcceptClient(_cit->name, g_config->GetEntryValueA(ET_CLIENTPASS));
		LoadString(g_hInst, IDS_EXCHANGE_ACCEPTED, g_szBuffer, STRING_BUFFER_SIZE);
	} else {
		g_exchanger->DeclineClient(_cit->name);
		LoadString(g_hInst, IDS_EXCHANGE_DENIED, g_szBuffer, STRING_BUFFER_SIZE);
	}
	os << ' ' << g_szBuffer;
	os.flush();
	g_logger->append(os.str().c_str());
}

/*
mn konfiguraci na zklad edit v dialogu s nastavenm
_hwnd - handler dialogu s nastavenm
_conf - aktuln konfigurace
*/
bool ChangeConfig(HWND _hwnd, CConfig *_conf)
{
	HWND edit;
	size_t count;
	TCHAR buffer[_conf->MAX_VALUE_LENGTH];
	for(int i = 0; i < ET_MAXENTRY; i++) {
		edit = GetDlgItem(_hwnd, IDC_SETTINGS_EDIT + i);
		if(!edit)
			continue;
		*(WORD *)buffer = _conf->MAX_VALUE_LENGTH - 1;
		count = SendMessage(edit, EM_GETLINE, 0, (LPARAM)buffer);
		buffer[count] = 0;
		if(!(count++) || !_conf->SetEntryValue((ENTRY_TYPE)i, buffer, count)) {
			SendMessage(edit, EM_SETSEL, 0, -1);
			_conf->Rollback();
			LoadString(g_hInst, IDS_SETUP_FAIL, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
			return false;
		}
	}
	_conf->Commit();
	LoadString(g_hInst, IDS_SETUP_SUCCESS, g_szBuffer, STRING_BUFFER_SIZE);
	g_logger->append(g_szBuffer);
	return true;
}

/*
vype dsledky zmn konfigurace do logu a vymae dan pznaky
_conf - aktuln konfigurace
*/
void PrintChanges(CConfig *_conf)
{
	const struct {
		unsigned int flag;
		int ids;
	} table[] = {
		{ CK_RENEW, IDS_CHANGE_RENEW },
		{ CK_REINIT, IDS_CHANGE_REINIT },
		{ CK_RESTART, IDS_CHANGE_RESTART }
	};

	for(int i = 0; i < 3; i++) {
		if(_conf->m_flags & table[i].flag) {
			LoadString(g_hInst, table[i].ids, g_szBuffer, STRING_BUFFER_SIZE);
			g_logger->append(g_szBuffer);
		}
		_conf->m_flags &= ~table[i].flag;
	}
	_conf->Commit();
}

/*
zobraz dialog oteven souboru, ppadn zmn hodnotu session v konfiguraci
_hwnd - handler hlavnho okna
_conf - aktuln konfigurace
*/
bool OpenConfig(HWND _hwnd, CConfig *_conf)
{
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = _hwnd;
	ofn.Flags = OFN_NOCHANGEDIR | OFN_FILEMUSTEXIST;
	TCHAR file[MAX_FILEPATH + 1];
	*file = 0;
	ofn.lpstrFile = file;
	ofn.nMaxFile = MAX_FILEPATH;
	ofn.lpstrFilter = TEXT("soubor konfigurace\0config.txt\0\0");
	ofn.nFilterIndex = 1;
	if(!GetOpenFileName(&ofn))
		return false;

	file[_tcslen(file) - 11] = 0;
	TCHAR *buf;
	buf = _tcsrchr(file, '\\');
	if(!buf)
		return false;
	buf = buf + 1;
	size_t count = _tcslen(buf) + 1;

	if(!_conf->SetEntryValue(ET_SESSION, buf, count))
		return false;

	return true;
}
