/*
nzev: manager.cpp
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#include "manager.h"
#include "interface.h"

bool CManager::Install(ENTRY_TYPE _et)
{
	TCHAR *name;
	INTERFACE_TYPE it;
	switch(_et) {
	case ET_TAPNAME:
		it = IT_TAP;
		break;
	case ET_LOOPBACKNAME:
		it = IT_LOOPBACK;
		break;
	case ET_BRIDGENAME:
		it = IT_BRIDGE;
		break;
	default:
		return false;
	}

	name = AddInterface(m_conf, it);
	bool ret = false;
	if(name) {
		ret = RenameNic(name, m_conf->GetEntryValue(_et));
		delete [] name;
	}
	return ret;
};

bool CManager::EnableServer()
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\enable_server.bat"),
		m_conf->GetEntryValue(ET_NICNAME),
		m_conf->GetEntryValue(ET_BRIDGENAME),
		m_conf->GetEntryValue(ET_PROTOCOL),
		m_conf->GetEntryValue(ET_BRIDGEADDRESS),
		m_conf->GetEntryValue(ET_BRIDGEMASK),
		m_conf->GetEntryValue(ET_OUTPORT),
		m_conf->GetEntryValue(ET_SERVERPORT)
	};
	return RunScript<true>(8, argv);
}

void CManager::DisableServer()
{
	RunScript<true>(TEXT("/K scripts\\disable_server.bat"));
}

void CManager::RunServer()
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\start_server.bat"),
		m_conf->GetEntryValue(ET_PATH),
		m_conf->GetEntryValue(ET_SESSION),
		m_conf->GetEntryValue(ET_TAPNAME),
		m_conf->GetEntryValue(ET_BRIDGEADDRESS),
		m_conf->GetEntryValue(ET_BRIDGEMASK),
		m_conf->GetEntryValue(ET_SERVERPORT),
		m_conf->GetEntryValue(ET_PROTOCOL),
		m_conf->GetEntryValue(ET_IPRANGE1),
		m_conf->GetEntryValue(ET_IPRANGE2)
	};
	RunScript<false>(10, argv);
}

void CManager::RunClient()
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\start_client.bat"),
		m_conf->GetEntryValue(ET_PATH),
		m_conf->GetEntryValue(ET_SESSION),
		m_conf->GetEntryValue(ET_TAPNAME),
		m_conf->GetEntryValue(ET_SERVERADDRESS),
		m_conf->GetEntryValue(ET_SERVERPORT),
		m_conf->GetEntryValue(ET_PROTOCOL)
	};
	RunScript<false>(7, argv);
}

bool CManager::NewServer()
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\new_server.bat"),
		m_conf->GetEntryValue(ET_PATH),
		m_conf->GetEntryValue(ET_SESSION),
		m_conf->GetEntryValue(ET_KEYSIZE),
		m_conf->GetEntryValue(ET_COUNTRY),
		m_conf->GetEntryValue(ET_CITY)
	};
	return RunScript<true>(6, argv);
}

bool CManager::NewClient()
{
	const TCHAR * const argv[] = {
		TEXT("/K scripts\\new_client.bat"),
		m_conf->GetEntryValue(ET_PATH),
		m_conf->GetEntryValue(ET_SESSION),
		m_conf->GetEntryValue(ET_CLIENTNAME),
		m_conf->GetEntryValue(ET_KEYSIZE),
		m_conf->GetEntryValue(ET_COUNTRY),
		m_conf->GetEntryValue(ET_CITY)
	};
	return RunScript<true>(7, argv);
}
