/*
nzev: server.cpp
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#include "server.h"

using namespace std;
using namespace Exchanger;

const timeval CServer::TIME_DELAY = { 0l, 1000l };

/*
inicializuje pipojovac vlkno a atributy
port - slo portu, na kterm bude naslouchat
*/
bool CServer::Init(unsigned short _port, bool (* _SignCSR)(const CConfig *, const char *), void (* _Notify)(NOTIFY_TYPE, const char *, const sockaddr_in *))
{
	m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(m_socket == INVALID_SOCKET) {
		m_socket = 0;
		return false;
	}
	setsockopt(m_socket, SOL_SOCKET, SO_OOBINLINE, FALSE, sizeof(BOOL));
	sockaddr_in addr;
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = INADDR_ANY;
	addr.sin_port = htons(_port);
	if(bind(m_socket, (sockaddr *)&addr, sizeof(addr)) == 0) {
		if(listen(m_socket, SOMAXCONN) == 0) {
			m_thread = CreateThread(NULL, 0, ServerThread, this, CREATE_SUSPENDED, &m_threadId);
			if(m_thread) {
				m_SignCSR = _SignCSR;
				m_Notify = _Notify;
				m_state = true;
				m_remain = FD_SETSIZE - 1;
				InitializeCriticalSection(&m_incs);
				ResumeThread(m_thread);
				return true;
			}
			CloseHandle(m_thread);
			m_thread = NULL;
		}
	}
	closesocket(m_socket);
	m_socket = 0;
	return false;
}

/*
destruktor, ukon vlkno a ukon vechny klienty
*/
CServer::~CServer()
{
	Exit();
}

/*
ukonuje pipojovac vlkno
*/
void CServer::Exit()
{
	m_state = false;
	if(m_thread) {
		DWORD dwSignaled = WaitForSingleObject(m_thread, INFINITE);
		if(dwSignaled == WAIT_OBJECT_0) {
			GetExitCodeThread(m_thread, &dwSignaled);
		}
		CloseHandle(m_thread);
		m_thread = NULL;
	}
	if(m_socket) {
		closesocket(m_socket);
		m_socket = 0;
	}
	list<CServerClient *>::iterator i;
	for(i = m_clients.begin(); i != m_clients.end(); i++) {
		delete *i;
	}
	m_clients.clear();

	MESSAGE_TYPE *msg;
	while(!m_inqueue.empty()) {
		msg = &m_inqueue.front();
		if(msg->accept)
			delete [] msg->passwd;
		delete [] msg->name;
		m_inqueue.pop();
	}

	DeleteCriticalSection(&m_incs);
}

void CServer::AcceptClient()
{
	sockaddr_in addr;
	int addrlen = sizeof(addr);
	SOCKET socket;

	socket = accept(m_socket, (sockaddr *)&addr, &addrlen);
	if(socket != INVALID_SOCKET) {
		CServerClient *client = new CServerClient(socket, addr, &m_cprov, m_conf, m_SignCSR);
		m_clients.push_back(client);
		m_remain--;
	}
}

void CServer::ProcessClient(list<CServerClient *>::iterator &_client)
{
	CServerClient *client = *_client;
	list<CServerClient *>::iterator buf = _client;
	_client++;
	if(!client->Process()) {
		switch(client->GetLastError()) {
		case ET_NOT_ALL_DATA:
			break;
		case ET_ATTEMPT:
			m_Notify(NT_ATTEMPT, client->GetName(), &client->GetAddress());
			break;
		case ET_COMPLETED:
			m_Notify(NT_COMPLETE, client->GetName(), &client->GetAddress());
		default:
			// zniceni klienta
			if(client->Shutdown()) {
				m_clients.erase(buf);
				delete client;
				m_remain++;
			}
			break;
		}
	}
}

void CServer::ProcessInputQueue()
{
	list<CServerClient *>::iterator i;
	char *name;
	CServerClient *client;

	EnterCriticalSection(&m_incs);
	while(!m_inqueue.empty()) {
		name = m_inqueue.front().name;
		for(i = m_clients.begin(); i != m_clients.end(); i++) {
			if(strcmp(name, (*i)->GetName()) == 0) {
				break;
			}
		}
		if(i != m_clients.end()) {
			client = *i;
			if(m_inqueue.front().accept) {
				// nastaveni spravneho hesla
				client->SetPasswd(m_inqueue.front().passwd);
				client->Accept();
				delete [] m_inqueue.front().passwd;
			} else {
				client->Decline();
			}
		}
		delete [] name;
		m_inqueue.pop();
	}
	LeaveCriticalSection(&m_incs);
}

void CServer::PostMessage(MESSAGE_TYPE *_msg)
{
	EnterCriticalSection(&m_incs);
	m_inqueue.push(*_msg);
	LeaveCriticalSection(&m_incs);
}

/*
funkce bc v samostatnm vlknu, pjm poadavky na spojen a vytv nov klientsk vlkna
param - objekt CServer, ke ktermu se vztahuje
*/
DWORD WINAPI Exchanger::ServerThread(LPVOID param)
{
	CServer &object = *(CServer *)param;
	list<CServerClient *>::iterator i;

	int result;
	fd_set sread, swrite;

	while(object.m_state) {
		object.ProcessInputQueue();

		FD_ZERO(&swrite);
		FD_ZERO(&sread);
		for(i = object.m_clients.begin(); i != object.m_clients.end(); i++) {
			(*i)->fdadd(&sread, &swrite);
		}
		FD_SET(object.m_socket, &sread);

		result = select(0, &sread, &swrite, NULL, &CServer::TIME_DELAY);
		if(SOCKET_ERROR == result || !result) {
			continue;
		}

		if(FD_ISSET(object.m_socket, &sread)) {
			object.AcceptClient();
			result--;
		}
		for(i = object.m_clients.begin(); result && i != object.m_clients.end();) {
			if((*i)->fdisset(&sread) || (*i)->fdisset(&swrite)) {
				object.ProcessClient(i);
				result--;
			} else
				i++;
		}
	}
	return TRUE;
}
