/*
nzev: server.h
autor: Pavel Hasenhrl
datum: 26.5.2009
verze: 1.0
*/

#ifndef SERVER_H
	#define SERVER_H

#include <list>
#include <queue>
#include "client.h"

namespace Exchanger {

enum NOTIFY_TYPE {
	NT_ATTEMPT,
	NT_COMPLETE
};

struct MESSAGE_TYPE {
	char *name;
	char *passwd;
	bool accept;
};

class CServer: public CPeer {
protected:
	SOCKET m_socket;
	HANDLE m_thread;
	DWORD m_threadId;
	bool m_state;
	const CConfig *m_conf;
	CCrypto m_cprov;

	int m_remain;

	static const timeval TIME_DELAY;// = { 0l, 1000l };

	std::list<CServerClient *> m_clients;

	CRITICAL_SECTION m_incs;
	std::queue<MESSAGE_TYPE> m_inqueue;

	void AcceptClient();
	void ProcessClient(std::list<CServerClient *>::iterator &_client);
	void ProcessAttempt(std::list<CServerClient *>::iterator &_client);
	void ProcessInputQueue();

	bool (* m_SignCSR)(const CConfig *, const char *);
	void (* m_Notify)(NOTIFY_TYPE, const char *, const sockaddr_in *);
	friend DWORD WINAPI ServerThread(LPVOID _param);
public:
	CServer(const CConfig *_conf): CPeer(), m_thread(NULL), m_socket(0), m_state(false), m_remain(FD_SETSIZE - 1), m_conf(_conf), m_cprov(_conf) {}
	virtual ~CServer();
	bool Init(unsigned short _port, bool (* _SignCSR)(const CConfig *, const char *), void (* _Notify)(NOTIFY_TYPE, const char *, const sockaddr_in *));
	void Exit();
	void PostMessage(MESSAGE_TYPE *_msg);
};

}

#endif
